package com.qianwankj.vt.vonage.flutter_vonage

import android.annotation.SuppressLint
import android.content.Context
import android.net.Uri
import android.os.Handler
import android.os.Looper
import com.nexmo.client.*
import com.nexmo.client.request_listener.NexmoApiError
import com.nexmo.client.request_listener.NexmoRequestListener
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

/**
 * 返回给flutter层信息
 */
class ResultEventInfo(private val type: String, private val info: String? = null) {
    override fun toString(): String {
        return "{\"type\":\"$type\",\"info\":\"${Uri.encode(info)}\"}"
    }
}

class CallUser(private val mid: String?, private val uid: String?, private val name: String?) {
    override fun toString(): String {
        return "{\"mid\":\"$mid\",\"uid\":\"$uid\",\"name\":\"$name\"}"
    }
}

/**
 * 1. 初始化
 * 2. 登录
 * 3. 拨打电话
 * 4. 发送信息
 */
object VonageUtils {

    private val mHandler = Handler(Looper.getMainLooper())
    private var mNexmoCall: NexmoCall? = null
    var eventSink: EventChannel.EventSink? = null

    private val mNexmoCallEventListener = object : NexmoCallEventListener {
        override fun onMemberStatusUpdated(
            callStatus: NexmoCallMemberStatus?,
            member: NexmoMember?
        ) {
            VtLogUtil.d("NexmoCallMemberStatus: $callStatus")

            if (callStatus == NexmoCallMemberStatus.COMPLETED || callStatus == NexmoCallMemberStatus.CANCELLED) {
                removeCurNexmoCall()
                // 已挂断
                callback(ResultEventInfo(Constants.CLOSE_CALL, ""))
            }
        }

        override fun onMuteChanged(newState: NexmoMediaActionState?, member: NexmoMember?) {
        }

        override fun onEarmuffChanged(newState: NexmoMediaActionState?, member: NexmoMember?) {
        }

        override fun onDTMF(dtmf: String?, member: NexmoMember?) {
        }
    }

    @SuppressLint("MissingPermission")
    fun onMethodCall(ctx: Context, call: MethodCall, result: MethodChannel.Result) {
        VtLogUtil.d("onMethodCall: ${call.method}")

        if (Constants.INIT == call.method) {
            VtLogUtil.open = call.argument<Boolean>("openLog") ?: true
            NexmoClient.Builder().build(ctx)

            // 默认设置连接监听
            NexmoClient.get().setConnectionListener { status, reason ->
                run {
                    val info = "{\"status\":\"$status\",\"reason\":\"$reason\"}"
                    VtLogUtil.d("ConnectionListener: $info")

                    callback(ResultEventInfo(Constants.CONNECTION_RESULT, info))
                }
            }

            // 默认添加来电监听
            NexmoClient.get().addIncomingCallListener {
                VtLogUtil.d("addIncomingCallListener: 接收到新的来电了")

                removeCurNexmoCall()

                mNexmoCall = it
                mNexmoCall?.addCallEventListener(mNexmoCallEventListener)

                // 来电待接通
                val cur = it.myMember // 当前用户
                val inUser = it.allMembers.find { itt -> itt.memberId != cur.memberId } // 来电的用户

                val sender = CallUser(inUser?.memberId, inUser?.user?.id, inUser?.user?.name)
                val receiver = CallUser(cur.memberId, cur.user.id, cur.user.name)
                val info = "{\"sender\":\"$sender\",\"receiver\":\"$receiver\"}"

                callback(ResultEventInfo(Constants.INCOMING_CALL, info))
            }

            result.success(Constants.INIT)
            return
        }

        if (Constants.LOGIN == call.method) {
            val phone = call.argument<String>("phone") ?: ""
            val jwtToken = call.argument<String>("jwtToken") ?: ""

            NexmoClient.get().login(jwtToken)

            result.success(Constants.LOGIN)
            return
        }

        if (Constants.LOGOUT == call.method) {
            NexmoClient.get().logout()

            result.success(Constants.LOGOUT)
            return
        }

        if (Constants.CLOSE == call.method) {
            // TODO 释放资源

            result.success(Constants.CLOSE)
            return
        }

        if (Constants.CREATE_CONVERSATION == call.method) {
            // TODO 创建会话

            result.success(Constants.CREATE_CONVERSATION)
            return
        }

        if (Constants.SEND_MESSAGE == call.method) {
            // TODO 发送消息
            val type = call.argument<String>("type") ?: ""
            val content = call.argument<String>("content") ?: ""

            result.success(Constants.SEND_MESSAGE)
            return
        }

        if (Constants.CALL_ANSWER == call.method) {
            // 接通
            mNexmoCall?.answer(object : NexmoRequestListener<NexmoCall> {
                override fun onSuccess(call: NexmoCall?) {
                    callback(ResultEventInfo(Constants.INCOMING_CALL_ANSWER, "1"))
                }

                override fun onError(apiError: NexmoApiError) {
                    callback(ResultEventInfo(Constants.INCOMING_CALL_ANSWER, "-1"))
                }
            })

            result.success(Constants.CALL_ANSWER)
            return
        }

        if (Constants.CALL_REJECT == call.method) {
            //  拒接
            mNexmoCall?.hangup(object : NexmoRequestListener<NexmoCall> {
                override fun onSuccess(call: NexmoCall?) {
                    mNexmoCall = null
                    callback(ResultEventInfo(Constants.INCOMING_CALL_REJECT, "1"))
                }

                override fun onError(apiError: NexmoApiError) {
                    callback(ResultEventInfo(Constants.INCOMING_CALL_REJECT, "-1"))
                }
            })

            result.success(Constants.CALL_REJECT)
            return
        }

        if (Constants.CALL == call.method) {
            // 拨号
            val type = call.argument<String>("type") ?: ""
            val fromNumber = call.argument<String>("fromNumber") ?: ""
            val toNumber = call.argument<String>("toNumber") ?: ""

            removeCurNexmoCall()

            NexmoClient.get().call(toNumber, NexmoCallHandler.fromString(type),
                object : NexmoRequestListener<NexmoCall> {
                    override fun onError(error: NexmoApiError) {
                        callback(ResultEventInfo(Constants.OUTGOING_CALL, "-1"))
                    }

                    override fun onSuccess(result: NexmoCall?) {
                        mNexmoCall = result

                        // 待接听
                        callback(ResultEventInfo(Constants.OUTGOING_CALL, "1"))

                        // 设置监听
                        mNexmoCall?.addCallEventListener(mNexmoCallEventListener)
                    }
                }
            )

            result.success(Constants.CALL)
            return
        }

        if (Constants.CALL_END == call.method) {
            // 挂断
            mNexmoCall?.hangup(object : NexmoRequestListener<NexmoCall> {
                override fun onSuccess(call: NexmoCall?) {
                    callback(ResultEventInfo(Constants.OUTGOING_CALL_HANGUP, "1"))
                }

                override fun onError(apiError: NexmoApiError) {
                    callback(ResultEventInfo(Constants.OUTGOING_CALL_HANGUP, "-1"))
                }
            })

            result.success(Constants.CALL_END)
            return
        }

        if (Constants.CALL_EVENT_MUTE == call.method) {
            // 是否静音
            val mute = call.argument<Boolean>("mute") ?: false
            mNexmoCall?.mute(mute)

            result.success(Constants.CALL_EVENT_MUTE)
            return
        }

        if (Constants.CALL_EVENT_SPEAK == call.method) {
            // 是否外放
            val speak = call.argument<Boolean>("speak") ?: false


            result.success(Constants.CALL_EVENT_SPEAK)
            return
        }

        if (Constants.CALL_EVENT_RECORD == call.method) {
            // 是否录音
            val record = call.argument<Boolean>("record") ?: false

            result.success(Constants.CALL_EVENT_RECORD)
            return
        }

        if (Constants.CALL_EVENT_DTMF == call.method) {
            // 发送指令
            val dtmf = call.argument<String>("dtmf") ?: ""
            mNexmoCall?.sendDTMF(dtmf, object : NexmoRequestListener<Void> {
                override fun onError(error: NexmoApiError) {
                }

                override fun onSuccess(result: Void?) {
                }

            })

            result.success(Constants.CALL_EVENT_DTMF)
            return
        }

        result.notImplemented()
    }

    // 移除现有状态
    private fun removeCurNexmoCall() {
        try {
            mNexmoCall?.removeCallEventListener(mNexmoCallEventListener)

        } catch (e: Exception) {
            VtLogUtil.e("removeCurNexmoCall: ${e.message}")
        }

        mNexmoCall = null
    }

    // 回传到flutter，需要在UI线程执行
    private fun callback(ev: ResultEventInfo) {
        mHandler.post {
            try {
                VtLogUtil.d("消息回调: $ev")

                eventSink?.success(ev.toString())
            } catch (e: Exception) {
                VtLogUtil.e("消息回调异常: ${e.message}")
            }
        }
    }

}